@@ -2,7 +2,7 @@ exams2html <- function(file, n = 1L, nsamp = NULL, dir = ".", template = NULL,
   name = NULL, quiet = TRUE, edir = NULL, tdir = NULL, sdir = NULL, verbose = FALSE,
   question = "<h4>Question</h4>", solution = "<h4>Solution</h4>",
   mathjax = FALSE, resolution = 100, width = 4, height = 4, svg = FALSE,
-  encoding = "", converter = NULL, ...)
+  encoding = "", envir = NULL, converter = NULL, ...)
 {
   ## for Rnw exercises use "ttm" converter and "plain" template,
   ## otherwise "pandoc" converter and "plain8" template
@@ -34,7 +34,7 @@ exams2html <- function(file, n = 1L, nsamp = NULL, dir = ".", template = NULL,
   ## create final .html exam
   rval <- xexams(file, n = n, nsamp = nsamp,
     driver = list(sweave = list(quiet = quiet, pdf = FALSE, png = !svg, svg = svg,
-      resolution = resolution, width = width, height = height, encoding = encoding),
+      resolution = resolution, width = width, height = height, encoding = encoding, envir = envir),
       read = NULL, transform = htmltransform, write = htmlwrite),
     dir = dir, edir = edir, tdir = tdir, sdir = sdir, verbose = verbose)
 
@@ -97,7 +97,7 @@ make_exams_write_html <- function(template = "plain", name = NULL,
 
   ## the link to mathjax
   mj_link <- paste('<script type="text/javascript"',
-    '  src="http://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML">',
+    '  src="https://mathjax.rstudio.com/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML">',
     '</script>', collapse = "\n")
   mathjax <- rep(mathjax, length.out = nt)
 
